/*
   Lagging Fibonacci Pseudo Random Generator for floating point types.

   The design of this template class is partially based on
   boost/random/lagged_fibonacci.hpp of the Boost library, see
   http://www.boost.org which is copyrighted by Jens Maurer 2000-2001
   and freely distributable under the Boost Software License:
   http://www.boost.org/LICENSE_1_0.txt.

   An instance of LFGReal generates floating point numbers of the type
   Real out of the range [0..1) of the lagged Fibonacci sequence

      x_n = (x_{n-p} + x_{n-q}) mod 1

   where p > q. The p seed values are extracted from the generator
   that is passed to the constructor using the () operator. The
   generator is expected to return values of the type UnsignedInteger
   in the range [0..2^wordWidth-1].

   This template expects the mathematical functions pow and fmod
   to be implemented for the Real type.

   Andreas F. Borchert
*/

#ifndef LFG_REAL_H
#define LFG_REAL_H

#include <cmath>

template<
   typename Real,
   typename UnsignedInteger,
   unsigned int wordWidth,
   unsigned int p, // p > q
   unsigned int q
> class LFGReal {
   public:
      typedef Real result_type;
      enum { long_lag = p };
      enum { short_lag = q };
      enum { lag_diff = p-q };

      template<typename Generator>LFGReal(Generator& gen) :
	    index(p) {
	 const UnsignedInteger mask(get_mask());
	 Real modulus = pow(Real(2), Real(wordWidth));
	 for (int i = 0; i < long_lag; ++i) {
	    values[i] = fmod((gen() & mask) / modulus, Real(1));
	 }
      }

      Real operator()() {
	 if (index >= long_lag) {
	    fill();
	 }
	 return values[index++];
      }

   private:
      Real values[p];
      unsigned int index;

      void fill() {
	 for (int i = 0; i < short_lag; ++i) {
	    Real val = values[i] + values[i + lag_diff];
	    if (val >= Real(1)) {
	       val -= Real(1);
	    }
	    values[i] = val;
	 }
	 for (int i = short_lag; i < long_lag; ++i) {
	    Real val = values[i] + values[i - short_lag];
	    if (val >= Real(1)) {
	       val -= Real(1);
	    }
	    values[i] = val;
	 }
	 index = 0;
      }

      static UnsignedInteger get_mask() {
	 UnsignedInteger m = 0;
	 for (int i = 0; i < wordWidth; ++i) {
	    m |= 1 << i;
	 }
	 return m;
      }
};

#endif

